; Nigel Goodwin 4 November 2001
; PWM routines incorporated in my original test program,
; in keeping with the supplied code I've added a macro for
; my delay routine, plus page select macros.
;
; Cybot test file
; v0.3
;
; David A Oram
; 28 October 2001
;
; PWM Routine based around techbot code by Jerry Merrill
;	with modifications for Cybot H-Bridge design
;	which has OFF/FWD & OFF/REV inputs for each motor
;	whereas techbot has OFF/ON & FWD/REV 

	list	P=PIC16F84
	include <P16F84.inc>
	__config H'3FF1'

;Program Variables stored in SRAM

	cblock	0x0C	;Start of SRAM
	d1, d2, del	;delay variables
	wsave, ssave	;Temp register store
	upcnt1, dwncnt1	;4 counters for PWM routine
	upcnt2, dwncnt2
	lftduty, rtduty	;Duty cycle for each motor
	mstatus		;motor activation register
			; Bit 0 = Left Reverse
			; Bit 1 = Left Forward
			; Bit 2 = Right Reverse
			; Bit 3 = Right Forward
	mmask		;mask for motor control
			; Used in PWM routine

	endc
;
;=====================================================	

;Motor Speed Constants

speed_stop	equ	0
speed_slow	equ	1
speed_2		equ	2
speed_3		equ	3
speed_4		equ	4
speed_half	equ	5
speed_6		equ	6
speed_7		equ	7
speed_8		equ	8
speed_9		equ	9
speed_full	equ	D'10'


onmsk	equ	B'11111111'	;Motor on Mask
offmsk1	equ	B'11111100'	;Motor1 off mask
offmsk2	equ	B'11110011'	;Motor2 off mask
motor	equ	PORTB	;Motor Port assignment

;	Decimal equvalents of motor assignment bits
lft_rev	equ	1
lft_fwd	equ	2	
rt_rev	equ	4
rt_fwd	equ	8

;Motor Direction Constants

forward		equ	lft_fwd+rt_fwd	;Move Forward
reverse		equ	lft_rev+rt_rev	;Move Reverse
rot_fwd_left	equ	rt_fwd
rot_fwd_right	equ	lft_fwd
rot_rev_left	equ	rt_rev
rot_rev_right	equ	lft_rev
spin_aclkwise	equ	rt_fwd+lft_rev
spin_clkwise	equ	lft_fwd+rt_rev

;=====================================================

;Macros

page1		macro
		bsf	STATUS,RP0
		endm

page0		macro
		bcf	STATUS,RP0
		endm

action	macro	direction,leftspeed,rightspeed

	local	spd=leftspeed+D'16'*rightspeed
	movlw	spd
	call	setspd
	
	movlw	direction
	movwf	mstatus

	endm

; Time delay macro, call wait [number of seconds] - max 25.5 seconds

wait	macro	variable
	movlw	variable*0x0a
	call	Delay

	endm


;=====================================================

;Start of Code

	org	0x00	;Reset vector

	nop
	goto	start

;motor control routine
;runs on each timer0 interrupt
;
;needs to be passed duty cycle for each motor
; in registers lftduty and rtduty
;Value of duty cycle is from 0 (stop) to 10 (Full speed)
;Also mstatus to contain details of motor directions
;
;Saves contents of w and status registers
;
	org	0x04	;interrupt vector

isr	movwf	wsave	;save accumulator
	movfw	STATUS
	movwf	ssave
	bcf	STATUS,RP0
			;now do first motor
	movfw	upcnt1
	btfsc	STATUS,Z
	goto	dodwn1
	decf	upcnt1,f
	goto	do2nd
dodwn1	movfw	dwncnt1
	btfsc	STATUS,Z
	goto	doover1
	decf	dwncnt1,f

	bcf	mmask,0	;motor 1 off
	bcf	mmask,1

	goto	do2nd
doover1	movfw	lftduty
	movwf	dwncnt1
	comf	dwncnt1,f
	movlw	11
	addwf	dwncnt1,f
	movfw	lftduty
	movwf	upcnt1
	btfss	STATUS,Z
	goto	mot1on
	goto	do2nd

mot1on	bsf	mmask,0	;turn on motor
	bsf	mmask,1

			;now do second motor
do2nd	movfw	upcnt2
	btfsc	STATUS,Z
	goto	dodown2
	decf	upcnt2,f
	goto	done
dodown2	movfw	dwncnt2
	btfsc	STATUS,Z
	goto	doover2
	decf	dwncnt2,f

	bcf	mmask,2	;turn motor 2 off
	bcf	mmask,3

	goto	done
doover2	movfw	rtduty
	movwf	upcnt2
	movwf	dwncnt2
	comf	dwncnt2,f
	movlw	11
	addwf	dwncnt2,f
			
	bsf	mmask,2	;motor 2 on
	bsf	mmask,3

done	movfw	mmask		;Output to motors
	andwf	mstatus,w
	movwf	motor

	bcf	INTCON,T0IF	;clear timer interrupt
	movfw	ssave		;restore w and status
	movwf	STATUS
	movfw	wsave
	retfie			;return from interrupt

;======================================================
; Subroutines
;	setspd	-Set Motor Speeds
;	Delay	-Delay 'W' 10ths of a second


;Set Motor Speeds
;	w register contains left and right speed
;		left speed in low nibble
;		right speed in high nibble
;
; This routine could probably be dispensed with
;	and left/right speeds passed directly

setspd	movwf	lftduty
	movwf	rtduty
	swapf	rtduty,f
	movlw	B'00001111'
	andwf	lftduty,f
	andwf	rtduty,f
	return


; call 100mS delay number of times in W
Delay	movwf	del
DelLoop	call 	Del100
	decfsz	del, f
	goto	DelLoop
	return

; 100mS delay
Del100	movlw	0x8d
	movwf	d1
Delay_00
	movlw	0xeb
	movwf	d2
Delay_01
	decfsz	d2, f
	goto	Delay_01
	decfsz	d1, f
	goto	Delay_00

			;25 cycles
	movlw	0x08
	movwf	d1
Delay_10
	decfsz	d1, f
	goto	Delay_10

			;1 cycles
	nop

			;4 cycles (including call)
	return


;======================================================

start	page0			;initialise i/o direction
	clrf	motor
	clrwdt
	page1
	movlw	B'11110000'	;Bits 0-3 are outputs
	movwf	TRISB
	movlw	1
	movwf	OPTION_REG	;prescaler /4
	movlw	B'11011111'
	andwf	OPTION_REG,f
	movlw	#0A0h
	movwf	INTCON
	page0
	goto	main

; Actions


main	action	forward,speed_half,speed_half
	wait	1
	action	forward,speed_stop,speed_stop
	wait	1

	action	reverse,speed_half,speed_half
	wait	1
	action	reverse,speed_stop,speed_stop
	wait	1

	action	rot_fwd_left,speed_half,speed_half
	wait	1
	action	forward,speed_stop,speed_stop
	wait	1

	action	rot_fwd_right,speed_half,speed_half
	wait	1
	action	forward,speed_stop,speed_stop
	wait	1

	action	spin_aclkwise,speed_half,speed_half
	wait	1
	action	forward,speed_stop,speed_stop
	wait	1

	action	spin_clkwise,speed_half,speed_half
	wait	1
	action	forward,speed_stop,speed_stop
	wait	1

	goto	main

	end
